/** GCODE viewer */

#gcode {
  .progress {
    width: 588px;

    .bar {
      -webkit-transition: width 0s linear;
      -moz-transition: width 0s linear;
      -o-transition: width 0s linear;
      transition: width 0s linear;
    }
  }

  .canvas_container {
    position: relative;
    &:hover,
    &:active {
      outline: 0;
    }
  }

  .layer-buttons {
    padding-top: 5px;
    padding-bottom: 7px;
  }

  #gcode_layer_slider {
    position: absolute;
    right: 0;
    top: 0;
    height: 568px;
    float: right;

    .slider-handle {
      width: 14px;
      height: 14px;
      margin-left: -3px;
      margin-top: -7px;
    }
  }

  #gcode_command_slider {
    .slider-handle {
      width: 14px;
      height: 14px;
      margin-left: -7px;
      margin-top: -3px;
    }
  }
}
