<div class="control-group" title="{{ _('FFMPEG command line to use')|edq }}">
    <label class="control-label" for="settings-webcamFfmpegCommandline">{{ _('FFMPEG command line') }}</label>
    <div class="controls">
        <input type="text" class="input-block-level" data-bind="value: webcam_ffmpegCommandline" id="settings-webcamFfmpegCommandline">
        <span class="help-block">{% trans %}
            You can adjust the full FFMPEG command line here that
            is used to generate the timelapse recordings, e.g. to add additional command line
            parameters as needed. Supported placeholders are:
            <code>{ffmpeg}</code> (configured path to ffmpeg), <code>{fps}</code> (configured framerate),
            <code>{input}</code> (input files), <code>{videocodec}</code> (configured video codec to use),
            <code>{threads}</code> (configured number of threads to use), <code>{bitrate}</code> (configured bitrate to use),
            <code>{containerformat}</code> (configured container format to use), <code>{filters}</code> (filter chain for flip/rotate & watermark),
            <code>{output}</code> (output file).
        {% endtrans %}</span>
    </div>
</div>
