<h3>{{ _('Hello!') }}</h3>

<p>{% trans %}
    Thank you for installing OctoPrint!
{% endtrans %}</p>
<p>{% trans %}
    This wizard will guide you through the final steps to get your OctoPrint instance all set up and ready to go. We'll
    get you printing in no time!
{% endtrans %}</p>
