<a id="navbar_show_settings" data-test-id="settings-open" title="{{ _('Settings')|edq }}" aria-label="{{ _('Settings')|edq }}" class="pull-right" href="#settings_dialog" data-bind="click: function() { $root.show(); }">
    <i class="fas fa-wrench"></i>
</a>
